@@ -10,6 +10,23 @@ Usage:
     This command prints the include path where Python_undef.h is located.
     python -m python_undef --help
     Display this help message.
+
+    You can also generate your project to shield the macros from your project's config.h:
+    ```python
+    import python_undef
+    if python_undef.generate_python_undef_header(
+        "path/to/your/project/config.h",
+        output_path="path/to/your/project/include",
+        project_name="MyProject",
+        main_header_macro="MYPROJECT_H",
+        main_header_name="myproject.h",
+        macro_need_header="MYPROJECT",
+        nonstandard_macro_rule=your_function
+    ):
+        print("good")
+    else:
+        print("bad")
+    ```
 """
 
 import os
@@ -17,8 +34,9 @@ import re
 import datetime
 import sys
 from pathlib import Path
+from typing import Callable
 
-MACRO_WRITELIST = [
+MACRO_WHITELIST = [
     "_W64",
     "_CRT_NONSTDC_NO_DEPRECATE",
     "_CRT_SECURE_NO_DEPRECATE"
@@ -63,27 +81,27 @@ def is_standard_python_macro(macro_name: str):
     Rules: Starts with Py, PY, _Py, _PY
     """
     standard_prefixes = ('Py', 'PY', '_Py', '_PY')
-    return macro_name.startswith(standard_prefixes) or macro_name in MACRO_WRITELIST
+    return macro_name.startswith(standard_prefixes) or macro_name in MACRO_WHITELIST
 
-def generate_undef_code(macro_name: str):
+def generate_undef_code(macro_name: str, macro_need_header: str="Py"):
     """Generate the code to undefine a macro."""
     return f"""#ifndef DONOTUNDEF_{macro_name}
 #ifdef {macro_name}
 #undef {macro_name}
-#ifdef _Py_FORWARD_DEFINE_{macro_name}
-#undef _Py_FORWARD_DEFINE_{macro_name}
-#pragma pop_macro("{macro_name}")
 #endif
+#ifdef _{macro_need_header}_FORWARD_DEFINE_{macro_name}
+#undef _{macro_need_header}_FORWARD_DEFINE_{macro_name}
+#pragma pop_macro("{macro_name}")
 #endif
 #endif
 
 """
 
-def generate_keep_code(macro_name: str):
+def generate_keep_code(macro_name: str, macro_need_header: str="Py"):
     """Generate the code to keep a macro."""
     return f"""#ifndef DONOTUNDEF_{macro_name}
 #ifdef {macro_name}
-#define _Py_FORWARD_DEFINE_{macro_name}
+#define _{macro_need_header}_FORWARD_DEFINE_{macro_name}
 #pragma push_macro("{macro_name}")
 #undef {macro_name}
 #endif
@@ -91,19 +109,27 @@ def generate_keep_code(macro_name: str):
 
 """
 
-def generate_python_undef_header(pyconfig_path: str, output_path: str|None=None):
+def generate_python_undef_header(pyconfig_path: str, / ,output_path: str|None=None, project_name: str="Python",
+                                 main_header_macro: str="Py_PYTHON_H", main_header_name: str="Python.h", macro_need_header: str="Py",
+                                 nonstandard_macro_rule: Callable[[str], bool]=is_standard_python_macro, inside_project: bool=False):
     """
-    Generate the Python_undef.h header file.
+    Generate the keep and undef header files based on your config.h.
     
     Args:
-        pyconfig_path: Path to pyconfig.h
-        output_path: Output file path, defaults to Python_undef.h in the current directory.
+        pyconfig_path: Path to your config.h file.
+        output_path: Output file path, defaults to undef and keep header in the current directory.
+        project_name: The name of the project, defaults to "Python".
+        main_header_macro: The macro that defines the main header, defaults to "Py_PYTHON_H".
+        main_header_name: The name of the main header file, defaults to "Python.h".
+        macro_need_header: The macro that defines the header that needs to be included, defaults to "Py".
+        nonstandard_macro_rule: A function that determines whether a macro is non-standard, defaults to is_standard_python_macro.
+        inside_project: Whether the code is inside the project, defaults to False.
     """
     if output_path is None:
         file_dir = os.path.dirname(os.path.abspath(__file__))
         include_dir = Path(file_dir) / 'include'
-        undef_output_path = str(include_dir / 'Python_undef.h')
-        keep_output_path = str(include_dir / 'Python_keep.h')
+        undef_output_path = str(include_dir / f'{project_name}_undef.h')
+        keep_output_path = str(include_dir / f'{project_name}_keep.h')
         if not include_dir.exists():
             try:
                 os.makedirs(f'{file_dir}/include')
@@ -115,8 +141,8 @@ def generate_python_undef_header(pyconfig_path: str, output_path: str|None=None)
             print(f"Error: Output path '{output_path}' is not a directory.", file=sys.stderr)
             return False
         include_dir = Path(output_path)
-        undef_output_path = str(include_dir / 'Python_undef.h')
-        keep_output_path = str(include_dir / 'Python_keep.h')
+        undef_output_path = str(include_dir / f'{project_name}_undef.h')
+        keep_output_path = str(include_dir / f'{project_name}_keep.h')
 
     # Read pyconfig.h
     try:
@@ -134,7 +160,8 @@ def generate_python_undef_header(pyconfig_path: str, output_path: str|None=None)
     all_macros = []
     invalid_macros = []
 
-    print("Analyzing pyconfig.h...")
+    filename = os.path.basename(pyconfig_path)
+    print(f"Analyzing {filename}...")
 
     for i, line in enumerate(lines, 1):
         macro_name = extract_macro_name(line)
@@ -142,11 +169,11 @@ def generate_python_undef_header(pyconfig_path: str, output_path: str|None=None)
             all_macros.append(macro_name)
 
             # New rule: any macro not starting with Py/PY/_Py/_PY and not ending with _H is considered non-standard
-            if not is_standard_python_macro(macro_name):
+            if not nonstandard_macro_rule(macro_name):
                 macros_to_undef.append(macro_name)
                 print(f"Line {i:4d}: Found non-standard macro '{macro_name}'")
         else:
-            # Check if line looks like a define but has invalid name
+            # Check if line looks like a definition but has invalid name
             line = line.strip()
             if line.startswith('#'):
                 m = re.match(r'^#\s*define\s+(\S+)', line)
@@ -160,65 +187,55 @@ def generate_python_undef_header(pyconfig_path: str, output_path: str|None=None)
 
     # Header section
     undef_header = f"""/*
- * Python_undef.h - Automatically generated macro undefinition header
- * 
+ * {project_name}_undef.h - Automatically generated macro undefinition header
+ *
  * This file is automatically generated from {os.path.basename(pyconfig_path)}
  * Contains macros that may need to be undefined to avoid conflicts with other libraries.
- * 
+ *
  * WARNING: This is an automatically generated file. Do not edit manually.
- * 
+{f''' *
  * Usage:
- *   #include <Python_keep.h>
- *   #include <Python.h>
- *   #include <Python_undef.h>
+ *   #include <{project_name}_keep.h>
+ *   #include <{main_header_name}>
+ *   #include <{project_name}_undef.h>
  *   #include <other_library_headers.h>
- * 
+ *''' if not inside_project else " *"}
  * To preserve specific macros, define before including this header:
  *   #define DONOTUNDEF_MACRO_NAME
- * 
- * Generation rules:
- *   - Macros starting with Py, PY, _Py, _PY are preserved (Python standard)
- *   - Macros ending with _H are preserved (header guards)
- *   - All other macros are undefined
- *   - Macro name validation uses Python's standard identifier checking
- * 
+ *
+ *
  * Generated from: {os.path.abspath(pyconfig_path)}
  * Generated at: {datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')}
  * Total valid macros found: {len(all_macros)}
  * Macros to undef: {len(macros_to_undef)}
  * Invalid macro names skipped: {len(invalid_macros)}
+ *
+ * Tool: python_undef
  */
 
-#ifndef PYTHON_UNDEF_H
-#define PYTHON_UNDEF_H
-
-#ifndef Py_PYTHON_H
-#  error "Python_undef.h must be included *after* Python.h"
+#ifndef {project_name.upper()}_UNDEF_H
+#define {project_name.upper()}_UNDEF_H
+{f'''
+#ifndef {main_header_macro}
+#  error "{project_name}_undef.h must be included *after* {main_header_name}"
 #endif
-
-/*
- * Platform Note:
- * - The COMPILER macro is primarily defined in pyconfig.h on Windows
- * - Other platforms define compiler info in Python/getcompiler.c
- * - This macro and others can conflict with libraries such as V8
- */
-
+''' if not inside_project else ""}
 """
 
     keep_header = f"""/*
- * Python_keep.h - Automatically generated macro keep header
+ * {project_name}_keep.h - Automatically generated macro keep header
  *
  * This file is automatically generated from {os.path.basename(pyconfig_path)}
  * Contains macros that are preserved to avoid conflicts with other libraries.
  *
  * WARNING: This is an automatically generated file. Do not edit manually.
- *
+{f''' *
  * Usage:
- *   #include <Python_keep.h>
- *   #include <Python.h>
- *   #include <Python_undef.h>
+ *   #include <{project_name}_keep.h>
+ *   #include <{main_header_name}>
+ *   #include <{project_name}_undef.h>
  *   #include <other_library_headers.h>
- *
+ *''' if not inside_project else " *"}
  * To preserve specific macros, define before including this header:
  *   #define DONOTUNDEF_MACRO_NAME
  *
@@ -226,29 +243,31 @@ def generate_python_undef_header(pyconfig_path: str, output_path: str|None=None)
  * Generated at: {datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')}
  * Total valid macros found: {len(all_macros)}
  * Macros to keep: {len(all_macros) - len(macros_to_undef)}
+ *
+ * Tool: python_undef
  */
 
-#ifndef PYTHON_KEEP_H
-#define PYTHON_KEEP_H
-
-#ifdef Py_PYTHON_H
-#error "Python_keep.h must be included *before* Python.h"
+#ifndef {project_name.upper()}_KEEP_H
+#define {project_name.upper()}_KEEP_H
+{f'''
+#ifdef {main_header_macro}
+#error "{project_name}_keep.h must be included *before* {main_header_name}"
 #endif
-
+''' if not inside_project else ""}
 """
 
     # Generate undef code sections
     undef_sections = []
     keep_selection = []
     for macro_name in macros_to_undef:
-        undef_sections.append(generate_undef_code(macro_name))
-        keep_selection.append(generate_keep_code(macro_name))
+        undef_sections.append(generate_undef_code(macro_name, macro_need_header))
+        keep_selection.append(generate_keep_code(macro_name, macro_need_header))
 
     # Footer
-    undef_footer = """#endif /* PYTHON_UNDEF_H */
+    undef_footer = f"""#endif /* {project_name.upper()}_UNDEF_H */
 """
 
-    keep_footer = """#endif /* PYTHON_KEEP_H */
+    keep_footer = f"""#endif /* {project_name.upper()}_KEEP_H */
 """
 
     # Write output
@@ -263,7 +282,7 @@ def generate_python_undef_header(pyconfig_path: str, output_path: str|None=None)
             f.write(keep_footer)
 
         print(f"\n{'='*60}")
-        print(f"Successfully generated: {undef_output_path} and {keep_output_path}")
+        print(f"Successfully generated: '{os.path.basename(undef_output_path)}' and '{os.path.basename(keep_output_path)}'")
         print(f"{'='*60}")
         print("Summary:")
         print(f"  - Total valid macro definitions: {len(all_macros)}")
@@ -287,13 +306,19 @@ def generate_python_undef_header(pyconfig_path: str, output_path: str|None=None)
 
         print(f"\nUsage Notes:")
         print(
-        f"  1. Include file \"Python_undef.h\" and \"Python_keep.h\" before including other library headers, "
-        "but \"Python_undef.h\" must be after '<Python.h>'.")
-        print(f"  2. Use DONOTUNDEF_XXX to protect macros that must be kept its defination in \"Python.h\".")
-        print(f"  3. Regenerate this file whenever rebuilding Python.")
-        
+            (
+                f"  1. Include file \"{project_name}_undef.h\" and \"{project_name}_keep.h\" before including other library headers"
+                f", but \"{project_name}_undef.h\" must be after '<{main_header_name}>'."
+            ) if not inside_project else 
+            (
+                f"  1. Include file \"{project_name}_keep.h\" before including \"{os.path.basename(pyconfig_path)}\" and "
+                f"\"{project_name}_undef.h\" after including the other your project headers file."
+            )
+        )
+        print(f"  2. Use DONOTUNDEF_XXX to protect macros that must be kept its definition in \"{main_header_name}\".")
+        print(f"  3. Regenerate this file whenever rebuilding {project_name}.")
+
         return True
-        
     except Exception as e:
         print(f"Error writing file: {e}", file=sys.stderr)
         return False
@@ -331,7 +356,7 @@ python -m python_undef --include
                         sys.exit(1)
                 else:
                     output_path = None
-                success = generate_python_undef_header(pyconfig_path, output_path)
+                success = generate_python_undef_header(str(pyconfig_path), output_path)
 
                 if success:
                     print(f"\n✅ Generation complete!")
